/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.


    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


	source:
		vector example

	revision history:
		Jan/01/1999 - Jukka Liimatta - initial revision
*/
#include <prmath/prmath.hpp>
#include <iostream>

using namespace prmath;
using namespace std;


namespace
{

	void Average()
	{
		const int num = 20;
		Vector3 array[num];

		Vector3 average(0,0,0);
		for ( int i=0; i<num; i++ )
			average += array[i];

		average /= num;
	}


	// simple triangle type for the example
	struct MyTri
	{
		Vector3		v[3];
		Vector3		normal;
	};


	void CalculateNormal(MyTri& tri)
	{
		Vector3 s = tri.v[0] - tri.v[1];
		Vector3 t = tri.v[0] - tri.v[2];
		tri.normal = Normalize( CrossProduct(s,t) );
	}


	void LerpTest()
	{
		Vector3 a( 0.0f, 0.0f, 0.0f );
		Vector3 b( 2.0f, 2.0f, 2.0f );

		Vector3 average = Lerp( a, b, 0.5f );
	}
	
} // namespace


int main()
{
	MyTri tri;
	tri.v[0] = Vector3(0,1,2);
	tri.v[1] = Vector3(1,0,0);
	tri.v[2] = Vector3(2,0,1);
	
	CalculateNormal( tri );
	Average();
	LerpTest();

	cout << "Vector test complete." << endl;

	return 0;
}
