/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.


    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


	source:
		alpha blending example

	revision history:
		Dec/07/2001 - Jukka Liimatta - initial revision
*/
#define PRCORE_IMPLEMENT_MAIN
#include <prcore/prcore.hpp>

using namespace prcore;


// =============================================================
// window declaration
// =============================================================

class TestWindow : public WindowBase
{
	public:

	TestWindow(int width, int height);
	~TestWindow();

	bool			EventMain();
	void			EventDraw();
	void			EventSize(int width, int height);
	void			EventKeyboard(int keycode, char charcode, bool press);
	
	private:

	Timer			timer;
	FrameBuffer		framebuffer;
	Bitmap			buffer;
	Bitmap			screen;
	Bitmap			sprite;
};


// =============================================================
// window implementation
// =============================================================

TestWindow::TestWindow(int width, int height)
{
	// open window
	Open(width,height,"TestWindow",WINDOW_NORMAL);
	
	// open framebuffer
	WinHandle handle = GetHandle();
	framebuffer.Open(handle);

	// create buffer bitmap
	buffer = Bitmap(width,height,PixelFormat(ARGB8888));

	// load screen/background bitmap
	screen.LoadImage("../screen.jpg");
	screen.ReformatImage(PixelFormat(ARGB8888));

	// load sprite bitmap
	sprite.LoadImage("../leaf.tga");
	sprite.ReformatImage(PixelFormat(ARGB8888));
}


TestWindow::~TestWindow()
{
	// close framebuffer
	framebuffer.Close();
}


bool TestWindow::EventMain()
{
	EventDraw();
	return true;
}


void TestWindow::EventDraw()
{
	float time = timer.GetTime();
	int x = (int)(sin(time) * 60 + 80);

	// "clear" buffer with screen/background bitmap
	buffer.BlitImage(screen);

	// setup alphablending
	SetAlphaFunc(SRCALPHA,SRCINVALPHA);
	SetConstantAlpha(0xff,0xff);

	// blend the sprite into the buffer
	AlphaBlend(x,10,buffer,sprite);

	framebuffer.BlitWrite(0,0,buffer);
	framebuffer.PageFlip();
}


void TestWindow::EventSize(int width, int height)
{
}


void TestWindow::EventKeyboard(int keycode, char charcode, bool press)
{
	if ( keycode==KEYCODE_ESC && press )
		MainBreak();
}


// =============================================================
// prmain()
// =============================================================

int prmain(int argc, char** argv)
{
	TestWindow window(400,300);
	window.MainLoop();
  
	return 0;
}
