/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRCORE_CODEC_HPP
#define PRCORE_CODEC_HPP


#include <prcore/array.hpp>


namespace prcore
{
	
	struct CodecInterface
	{
		int				count;
		const char**	extension;
	};


	template <typename T>
	class CodecManager
	{
		public:

		int				GetInterfaceCount() const;
		const T*		GetInterface(int index) const;
		const T*		FindInterface(const char* ext) const;
		void			RegisterInterface(const T& codec);

		private:

		Array<T>		mCodecArray;
	};
	
	
	// inline methods

	template <typename T>
	int CodecManager<T>::GetInterfaceCount() const
	{
		return mCodecArray.GetSize();
	}


	template <typename T>
	const T* CodecManager<T>::GetInterface(int index) const
	{
		return &mCodecArray[index];
	}


	template <typename T>
	const T* CodecManager<T>::FindInterface(const char* ext) const
	{
		if ( !ext )
			return NULL;

		// iterate through all codecs
		for ( int i=0; i<mCodecArray.GetSize(); ++i )
		{
			const T& codec = mCodecArray[i];

			// search all extensions in currect codec
			for ( int j=0; j<codec.count; ++j )
			{
				if ( !stricmp(codec.extension[j],ext) )
					return &codec;
			}
		}

		return NULL;
	}


	template <typename T>
	void CodecManager<T>::RegisterInterface(const T& codec)
	{
		mCodecArray.PushBack(codec);
	}

} // namespace prcore


#endif
