/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRCORE_EXCEPTION_HPP
#define PRCORE_EXCEPTION_HPP


#include <prcore/string.hpp>
#include <prcore/bitmap.hpp>


namespace prcore
{

	class Exception
	{
		public:

		Exception();
		Exception(const char* method, const char* message, const char* filename = NULL);
		Exception(const Exception& other);
virtual	~Exception();

		void			operator = (const Exception& other);

		void			DrawDialog();					// draw the exception dialog
		const String&	GetFilename() const;			// file which did throw the exception
		const String&	GetMethod() const;				// method/function which did throw the exception
		const String&	GetMessage() const;				// user message attached to the exception

static	bool		SetLogo(const class Bitmap& logo);	// set the user's own logo (128x94x256) for the exception dialog

		private:

		String			mFilename;
		String			mMethod;
		String			mMessage;
	};


	// exception macro

	#define PRCORE_EXCEPTION(method,message) throw prcore::Exception(method,message,__FILE__);

} // namespace prcore


#endif
