/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRCORE_FILESTREAM_HPP
#define PRCORE_FILESTREAM_HPP


#include <prcore/stream.hpp>
#include <prcore/string.hpp>
#include <prcore/codec.hpp>
#include <prcore/array.hpp>


namespace prcore
{

	class FileStream : public Stream
	{
		public:
		
		FileStream(const char* filename, AccessMode mode = READ);
		~FileStream();
		
		int			Seek(int delta, SeekMode mode);
		int			Read(void* target, int bytes);
		int			Write(const void* source, int bytes);
		int			GetOffset() const;
		int			GetSize() const;
		bool		IsOpen() const;
		bool		IsEOS() const;

static	void		SetPassword(const char* password);
static	void		SetBasePath(const char* path);
static	String		GetBasePath();

		private:
		
		enum { CACHE_BITS = 12, CACHE_SIZE = (1 << CACHE_BITS) };
		
		Stream*		stream;
		bool		iscache;
		char		cache[CACHE_SIZE];
		int			offset;
		int			page;
	};
	
	
	class FileIndex
	{
		public:

		FileIndex();
		FileIndex(const char* userpath);
		~FileIndex();

		void			CreateIndex(const char* userpath);

		int				GetFileCount() const;
		int				GetFolderCount() const;
		const char*		GetFileName(int index) const;
		const char* 	GetFolderName(int index) const;	

		private:

		Array<String>	mFileArray;
		Array<String>	mFolderArray;
	};


	struct FileCodec : public CodecInterface
	{
		Stream*		(*stream)(Stream* archive, const char* filename, Stream::AccessMode mode, void* password);
		void		(*index)(Stream* archive, const char* path, Array<String>& filename, Array<String>& foldername);
		bool		(*isfolder)(Stream* archive, const char* path);
	};


	class FileCodecManager : public CodecManager<FileCodec>
	{
		public:

		static FileCodecManager manager;
		
		private:

		FileCodecManager();
		FileCodecManager(const FileCodecManager&);
		FileCodecManager& operator = (const FileCodecManager&);
	};

} // namespace prcore


#endif
