/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRCORE_STRING_HPP
#define PRCORE_STRING_HPP


#include <assert.h>
#include <iostream>
#include <prcore/configure.hpp>


namespace prcore
{

	class String
	{
		public:

		String();
		String(const String& s);
		String(const char* text);
		~String();

inline	String& 	operator << (const char* text);
inline	String& 	operator << (const unsigned char* text);
inline	String& 	operator << (const signed char* text);
inline	String& 	operator << (const char c);
inline	String& 	operator << (const unsigned char uc);
inline	String& 	operator << (const signed char sc);
      	String& 	operator << (const short s);
      	String& 	operator << (const unsigned short us);
      	String& 	operator << (const int i);
      	String& 	operator << (const unsigned int ui);
      	String& 	operator << (const long l);
      	String&		operator << (const unsigned long ul);
     	String& 	operator << (const float f);

		String&		operator = (const String& string);
		String&		operator = (const char* text);
		String&		operator += (const String& string);
		String&		operator += (const char* text);
		String		operator + (const String& string) const;
		String		operator + (const char* text) const;
		bool		operator == (const String& string) const;
		bool		operator == (const char* text) const;
		bool		operator != (const String& string) const;
		bool		operator != (const char* text) const;
inline	char&		operator [] (int index);
inline	const char&	operator [] (int index) const;
inline 				operator char* ();
inline 				operator const char* () const;

		void 		PrintText(const char* text, ...);
inline	bool		IsEmpty() const;
inline	int			GetLength() const;
		String		SubString(int offset, int length) const;

		private:

		char*		mData;
		int			mLength;
		
		String(char* text, int length);
	};


	// inline methods

	inline String& String::operator << (const char* text)
	{
		assert( text != NULL );
		return operator += (text);
	}

	inline String& String::operator << (const unsigned char uc)
	{
		char text[2];
		text[0] = uc;
		text[1] = 0;
		return operator += (text);
	}

	inline String& String::operator << (const char c)
	{ 
		return operator << ((const unsigned char)c); 
	}

	inline String& String::operator << (const signed char sc)
	{ 
		return operator << ((const unsigned char)sc);
	}

	inline String& String::operator << (const unsigned char* uc)
	{ 
		return operator << ((const char*)uc);
	}

	inline String& String::operator << (const signed char* sc)
	{ 
		return operator << ((const char*)sc);
	}

	inline char& String::operator [] (int index)
	{
		assert( index>=0 && index < mLength );
		return mData[index];
	}

	inline const char& String::operator [] (int index) const
	{
		assert( index>=0 && index < mLength );
		return mData[index];
	}

	inline String::operator char* ()
	{
		return mData ? mData : (char*)"";
	}

	inline String::operator const char* () const
	{
		return mData ? mData : "";
	}

	inline bool String::IsEmpty() const
	{
		return mData ? false : true;
	}

	inline int String::GetLength() const
	{
		return mLength;
	}

	// std::ostream global inlines

	inline std::ostream& operator << (std::ostream& out, String& s)
	{
		out << (const char*)s;
		return out;
	}

	inline std::ostream& operator << (std::ostream& out, const String& s)
	{
		out << (const char*)s;
		return out;
	}

} // namespace prcore


#endif
