/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRCORE_WINDOW_HPP
#define PRCORE_WINDOW_HPP


#include <prcore/configure.hpp>
#include <prcore/refcount.hpp>
#include <prcore/rect.hpp>


namespace prcore
{

	enum 
	{
		// window style flags
		WINDOW_SIZE			= 0x001, // window has resize button
		WINDOW_CLOSE		= 0x002, // window has close button
		WINDOW_MINIMIZE		= 0x004, // window has minimize button
		WINDOW_FRAME		= 0x008, // window has frame
		WINDOW_TITLE		= 0x010, // window has title
		WINDOW_QUITONCLOSE	= 0x080, // terminate application on close
		WINDOW_USES3D		= 0x100, // window supports 3d acceleration
	       
		// pre-defined styles
		WINDOW_FULLSCREEN	= 0x200 | WINDOW_QUITONCLOSE,
		WINDOW_NORMAL		= 0xffffffff
	};


	class WindowBase : public RefCount
	{
		public:

		WindowBase();
		WindowBase(WinHandle parent);
		~WindowBase();

		bool			Open(int width, int height, const char* name, int style);
		bool			Close();
		void			Rename(const char* name);
		void			Resize(int width, int height);
		void			Move(int x, int y);
		void			ShowCursor();
		void			HideCursor();
		void			ShowWindow();
		void			HideWindow();

		void			MainLoop();
		void			MainBreak();

virtual	bool			EventMain();
virtual	void			EventDraw();
virtual	void			EventSize(int width, int height);
virtual	void			EventKeyboard(int keycode, bool press);
virtual	void			EventKeyboard(int keycode, char charcode, bool press);
virtual	void			EventFocus(bool enter);
virtual	void			EventDropFile(const char* filename);
virtual	void			EventClose();

		bool			IsOpen() const;
		int				GetStyle() const;
		WinHandle		GetHandle() const;
		Rect			GetRect() const;
		float			GetAspect() const;
		bool			IsDown(int keycode) const;
		
		void			DispatchEventKeyboard(int keycode, char charcode, bool press);

		protected:

		WinHandle		mWinParent;
		WinHandle		mWinHandle;
		int				mWinStyle;
		
		private:

		bool			mMainBreak;
		bool			mKeyDown[256];
	};

} // namespace prcore


#endif
