/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRMATH_BOX_HPP
#define PRMATH_BOX_HPP


#include <cfloat>


namespace prmath
{

	struct Box
	{
		Vector3		vmin;
		Vector3		vmax;

		Box();
		Box(const Vector3& umin, const Vector3& umax);
		Box(const Box& a, const Box& b);

		void		UpdateBox(const Vector3& umin, const Vector3& umax);
		void		UpdateBox(const Vector3& v);

		Vector3		GetCenter() const;
		Vector3		GetSize() const;
		bool		IsInside(const Vector3& point) const;
		void		GetVertexArray8(Vector3 vertex[]) const;
	};


	// inline methods

	inline Box::Box()
	: vmin(FLT_MAX,FLT_MAX,FLT_MAX),vmax(-FLT_MAX,-FLT_MAX,-FLT_MAX)
	{
	}

	inline Box::Box(const Vector3& umin, const Vector3& umax)
	: vmin(umin),vmax(umax)
	{
	}

	inline void Box::UpdateBox(const Vector3& umin, const Vector3& umax)
	{
		if ( umin.x < vmin.x) vmin.x = umin.x;
		if ( umin.y < vmin.y) vmin.y = umin.y;
		if ( umin.z < vmin.z) vmin.z = umin.z;
		if ( umax.x > vmax.x) vmax.x = umax.x;
		if ( umax.y > vmax.y) vmax.y = umax.y;
		if ( umax.z > vmax.z) vmax.z = umax.z;
	}

	inline void Box::UpdateBox(const Vector3& v)
	{
		if ( v.x < vmin.x ) vmin.x = v.x; else if ( v.x > vmax.x ) vmax.x = v.x;
		if ( v.y < vmin.y ) vmin.y = v.y; else if ( v.y > vmax.y ) vmax.y = v.y;
		if ( v.z < vmin.z ) vmin.z = v.z; else if ( v.z > vmax.z ) vmax.z = v.z;
	}

	inline Vector3 Box::GetCenter() const
	{
		return (vmin + vmax) * 0.5f;
	}

	inline Vector3 Box::GetSize() const
	{
		return vmax - vmin;
	}

	inline bool Box::IsInside(const Vector3& point) const
	{
		if ( point.x < vmin.x ) return false;
		if ( point.y < vmin.y ) return false;
		if ( point.z < vmin.z ) return false;
		if ( point.x > vmax.x ) return false;
		if ( point.y > vmax.y ) return false;
		if ( point.z > vmax.z ) return false;
		return true;
	}

	inline void Box::GetVertexArray8(Vector3 vertex[]) const
	{
		vertex[0].x = vmin.x; vertex[0].y = vmin.y; vertex[0].z = vmin.z;
		vertex[1].x = vmax.x; vertex[1].y = vmin.y; vertex[1].z = vmin.z;
		vertex[2].x = vmin.x; vertex[2].y = vmax.y; vertex[2].z = vmin.z;
		vertex[3].x = vmax.x; vertex[3].y = vmax.y; vertex[3].z = vmin.z;
		vertex[4].x = vmin.x; vertex[4].y = vmin.y; vertex[4].z = vmax.z;
		vertex[5].x = vmax.x; vertex[5].y = vmin.y; vertex[5].z = vmax.z;
		vertex[6].x = vmin.x; vertex[6].y = vmax.y; vertex[6].z = vmax.z;
		vertex[7].x = vmax.x; vertex[7].y = vmax.y; vertex[7].z = vmax.z;
	}
	
} // namespace prmath


#endif
