/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.
*/
#ifndef PRMATH_INTERSECT_HPP
#define PRMATH_INTERSECT_HPP


namespace prmath
{

	// functions

	bool	IntersectConeSphere(const Cone& cone, const Sphere& sphere);
	bool	IntersectBoxSphere(const Box& box, const Sphere& sphere);
	bool	IntersectRaySphere(const Line& ray, const Sphere& sphere, float& enter);
	bool	IntersectRaySphere(const Line& ray, const Sphere& sphere, float& enter, float& leave);
	bool	IntersectRayPlane(const Line& ray, const Plane& plane, float& enter);
	bool	IntersectRayTriangle(const Line& ray, const Vector3& v0, const Vector3& v1, const Vector3& v2, float& enter);
	bool	IntersectRayBox(const Line& ray, const Box& box, float& enter, float& leave);
	bool	PlaneIntersect(const Plane& p0, const Plane& p1, Vector3& point, Vector3& direction);
	bool	PlaneIntersect(const Plane& p0, const Plane& p1, const Plane& p2, Vector3& point);
	bool	PointInsideTriangle(const Vector3& v0, const Vector3& v1, const Vector3& v2, const Vector3& point);
	float	GetTriangleArea(float edge0, float edge1, float edge2);

	// inline functions

	inline bool IntersectRaySphere(const Line& ray, const Sphere& sphere)
	{
		float temp;
		return IntersectRaySphere(ray,sphere,temp);
	}

	inline bool IntersectRayPlane(const Line& ray, const Plane& plane)
	{
		float temp;
		return IntersectRayPlane(ray,plane,temp);
	}

	inline bool IntersectRayTriangle(const Line& ray, const Vector3& v0, const Vector3& v1, const Vector3& v2)
	{
		float temp;
		return IntersectRayTriangle(ray,v0,v1,v2,temp);
	}

	inline bool IntersectRayBox(const Line& ray, const Box& box)
	{
		float temp;
		return IntersectRayBox(ray,box,temp,temp);
	}

} // namespace prmath


#endif
