/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.


    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


	source: 
		cpu capabilities

	revision history:
		May/10/1998 - Jukka Liimatta - Initial revision
*/
#include <prcore/configure.hpp>
#include <prcore/cpu.hpp>

using namespace prcore;


// =================================================
// flags
// =================================================

namespace
{
	
	const uint32 CPU_INIT    = ~0;
	const uint32 CPU_NONE    = 0;
	const uint32 CPU_MMX     = 0x0001;
	const uint32 CPU_3DNOW   = 0x0002;
	const uint32 CPU_SSE     = 0x0004;
	const uint32 CPU_SSE2    = 0x0008;
	
} // namespace


// =================================================
// extern
// =================================================

#ifdef PRCORE_X86_SIMD
extern "C"
{
	extern uint32 x86_cpu_caps();
};
#else
	static uint32 x86_cpu_caps() {return CPU_NONE;}
#endif


// =================================================
// implementation
// =================================================

static inline uint32 cpuflags()
{
	// first time this function is called,
	// the cpu flags are queried
	static uint32 flags = CPU_INIT;
	if ( flags == CPU_INIT )
	{
		flags = x86_cpu_caps();
	}
	return flags;
}


// =================================================
// API
// =================================================

bool prcore::IsMMX()
{
	return (cpuflags() & CPU_MMX) != 0;
}


bool prcore::IsSSE()
{
	return (cpuflags() & CPU_SSE) != 0;
}


bool prcore::IsSSE2()
{
	return (cpuflags() & CPU_SSE2) != 0;
}


bool prcore::Is3DNOW()
{
	return (cpuflags() & CPU_3DNOW) != 0;
}

