/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.


    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


	source:
		InputDevice

	revision history:
		Jun/01/2000 - Jukka Liimatta - initial revision
		Jan/24/2001 - Jukka Liimatta - renaissance build
*/
#include <prcore/configure.hpp>
#include <prcore/inputdevice.hpp>

using namespace prcore;


// =================================================
// InputBase
// =================================================

int InputBase::GetAxisCount() const
{
	return 0;
}


int InputBase::GetButtonCount() const
{
	return 0;
}


float InputBase::GetAxis(int index)
{
	return 0.f;
}


bool InputBase::GetButton(int index)
{
	return false;
}


// =================================================
// InputDevice
// =================================================

InputDevice::InputDevice()
{
}


InputDevice::~InputDevice()
{
}


InputDevice::BindIO* InputDevice::FindBindIO(Array<InputDevice::BindIO>& array, int index)
{
	for ( int i=0; i<array.GetSize(); i++ )
	{
		BindIO& bind = array[i];
		if ( bind.dest == index )
		{
			return &bind;
		}
	}
	return NULL;
}


void InputDevice::BindAxis(int dest, int src, InputBase& input)
{
	Array<BindIO>& array = axisbinds;
	BindIO* ptr = FindBindIO( array, dest );

	if ( !ptr )
	{
		BindIO bind;

		bind.dest = dest;
		bind.src = src;
		bind.input = &input;

		array.PushBack( bind );
	}
	else
	{
		BindIO& bind = *ptr;

		bind.dest = dest;
		bind.src = src;
		bind.input = &input;
	}
}


void InputDevice::BindButton(int dest, int src, InputBase& input)
{
	Array<BindIO>& array = buttonbinds;
	BindIO* ptr = FindBindIO( array, dest );

	if ( !ptr )
	{
		BindIO bind;

		bind.dest = dest;
		bind.src = src;
		bind.input = &input;

		array.PushBack( bind );
	}
	else
	{
		BindIO& bind = *ptr;

		bind.dest = dest;
		bind.src = src;
		bind.input = &input;
	}
}


int InputDevice::GetAxisCount() const
{
	return int( axisbinds.GetSize() );
}


int InputDevice::GetButtonCount() const
{
	return int( buttonbinds.GetSize() );
}


float InputDevice::GetAxis(int index)
{
	BindIO* ptr = FindBindIO( axisbinds, index );
	if ( ptr )
	{
		BindIO& bind = *ptr;
		return bind.input->GetAxis( bind.src );
	}

	return 0.f;
}


bool InputDevice::GetButton(int index)
{
	BindIO* ptr = FindBindIO( buttonbinds, index );
	if ( ptr )
	{
		BindIO& bind = *ptr;
		return bind.input->GetButton( bind.src );
	}

	return false;
}

