/*
	Twilight Prophecy SDK
	A multi-platform development system for virtual reality and multimedia.

	Copyright (C) 1997-2002 Twilight 3D Finland Oy Ltd.


    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


	source:
		Exception Linux implementation
		
	revision history:
		???/??/2001 - Edmond Cote - initial revision
		09/06/2002  - Edmond Cote - 288 build
*/

#include <prcore/prcore.hpp>
using namespace prcore;


static char ExFilename[] = "UnknownFile";
static char ExMethod[] = "UnknownClass";
static char ExMessage[] = "Reason not specified.";

Exception::Exception()
{
  mFilename = ExFilename;
  mMethod = ExMethod;
  mMessage = ExMessage;
}

Exception::Exception(const char* method, const char* message, const char* file)
{
  mFilename = file ? file : ExFilename;
  mMethod = method ? method : ExMethod;
  mMessage = message ? message : ExMessage;
}

Exception::Exception(const Exception& e)
{
  *this = e;
}

void Exception::operator = (const Exception& e)
{
  mFilename = e.mFilename;
  mMethod = e.mMethod;
  mMessage = e.mMessage;
}


const String& Exception::GetFilename() const
{
  return mFilename;
}


const String& Exception::GetMethod() const
{
  return mMethod;
}

const String& Exception::GetMessage() const
{
  return mMessage;
}


Exception::~Exception()
{
}

bool Exception::SetLogo(const class Bitmap& logo)
{
  // I'm not supporting this quite yet
  return true;
}

void Exception::DrawDialog()
{
  // right now this passes to stdout


  cout << "Prophecy3D / Multimedia SDK Exception Handler\n"
       << "---------------------------------------------\n"
       << "Filename: " << mFilename << endl
       << "Method: " << mMethod << endl
       << "Message: " << mMessage << endl;
       
}
